unit NBSUPort;{NetBios support}

interface

uses dos;

type

  tName = array [0..15] of char;
  tNCB = record
    command : BYTE;                 {command code (see below)}
    retcode : BYTE;                     {return code}
    lsn     : BYTE;                 {local session number (LSN)}
    num     : BYTE;                 {"ncbnum" datagram table entry from ADD
NAME}
    buffer  : pointer;                 {-> I/O buffer}
    length  : WORD;                 {length of data in buffer}
    callname: tName;                 {remote system to call}
    name    : tName;                 {network name of local machine}
    RTo     : Byte;                 {receive timeout in 1/2 seconds}
    STo     : Byte;                 {send timeout in 1/2 seconds}
    Post    : Pointer;                 {-> FAR post handler }
    lananum : BYTE;                 {network adapter number on which to execute
command}
        {00h-03h IBM NetBIOS specs}
        {F0h-FFh Eicon NABios interface (see also INT 7B"Eicon")}
    cmdcptl : BYTE;                 {command completion code (see returned
status above)}
    reserved: array[0..13]of BYTE; {reserved for network card}
  end;

  S16 = String[16];

  TStatusBuf = record
    Number: array [0..5] of Byte;
    a: array [1..52] of Byte;
  end;

function AddName (Local: string; var NCB: TNCB): Integer;
function DeleteName (Local: string; var NCB: TNCB): Boolean;
procedure Call (Local, Remote: string; var NCB: TNCB; var ALSN, ANum: Integer);
function Listen (Local, Remote: string; var NCB: TNCB): Integer;
function DelayedListen (Local, Remote: string; Proc: Pointer; var NCB: TNCB):
Boolean;
function Send (ALSN: Integer; var Buf; Len: Integer; var NCB: TNCB): Integer;
function DelayedSend (ALSN: Integer; var Buf; Len: Integer; Proc: Pointer; var
NCB: TNCB): Boolean;
function SendDatagram (const Remote: string; var Buf; Len: Integer; var NCB:
TNCB): Boolean;
function SendBroadDatagram (var Buf; Len: Integer; var NCB: TNCB): Boolean;
function Receive (ALSN: Integer; var Buf; Len: Integer; var NCB: TNCB): Integer;
function DelayedReceive (ALSN: Integer; var Buf; Len: Integer; Proc: Pointer;
var NCB: TNCB): Integer;
function ReceiveDatagram (var Buf; Len: Integer; Proc: Pointer; var NCB: TNCB):
Boolean;
function ReceiveBroadDatagram (var Buf; Len: Integer; Proc: Pointer; var NCB:
TNCB): Boolean;
function HangUp (LSN: Integer; var NCB: TNCB): Boolean;
function Cancel(var NCB: TNCB): Boolean;
function ConvStr(D: TName): String;

implementation

function NBInt (cmD: Byte; var NCB: TNCB): Byte;
var R: RegisterS;
begin
  ncb.command := cmd;
  r.es := seg(ncb);
  r.bx := ofs(ncb);
  intr($5c, r);
  NBInt := r.al;
end;

procedure NameCopy (S: string; var D: TName);
begin
  FillChar (D, SizeOf (D), 0);
  Move (S [1], D, Length (S));
end;

function ConvStr(D: TName): String;
var      S: String;
         I: Integer;
begin
  I := 1;
  while D[I] <> #0 do Inc(I);
  S[0] := Chr(I);
  Move (D, S[1], SizeOf(D));
  ConvStr := S;
end;

function Cancel(var NCB: TNCB): Boolean;
var LNCB: TNCB;
begin
LNCB := NCB;
asm
  mov ax, word ptr NCB
  mov word ptr LNCB.Buffer, ax
  mov ax, word ptr NCB[2]
  mov word ptr LNCB.Buffer[2], ax
end;
Cancel := NBInt ($35, LNCB) = 0;
end;

function ADDNAME;
begin
  NameCopy (Local, NCB.Name);
  if NBInt ($30, NCB) = 0
  then AddName := NCB.Num
  else AddName := 0;
end;

function DeleteName;
begin
  NameCopy (Local, NCB.Name);
  DeleteName := NBInt ($31, NCB) <> 0;
end;

procedure Call;
begin
  with NCB do
  begin
    NameCopy (Remote, CallName);
    NameCopy (Local, Name);
    STO := 10;
    RTO := 10;
    if NBInt ($10, NCB) = 0
    then
    begin
      ALSN := LSN;
      ANum := Num;
    end
    else
      ALSN := 0;
  end;
end;

function LISTEN;
begin
  with NCB do
  begin
    NameCopy (Remote, CallName);
    NameCopy (Local, Name);
    STO := 10;
    RTO := 10;
    if NBInt ($11, NCB) = 0
    then LISTEN := LSN
    else LISTEN := 0;
  end;
end;

function DELAYEDLISTEN;
begin
  with NCB do
  begin
    NameCopy (Remote, CallName);
    NameCopy (Local, Name);
    STO := 10;
    RTO := 10;
    Post := Proc;
    DELAYEDLISTEN := NBInt ($91, NCB) = 0;
  end;
end;

function SEND;
begin
  with NCB do
  begin
    LSN := aLSN;
    Buffer := @Buf;
    Length := Len;
    if NBInt ($14, NCB) = 0
    then SEND := Len
    else SEND := 0;
  end;
end;

function DELAYEDSEND;
begin
  with NCB do
  begin
    LSN := aLSN;
    Buffer := @Buf;
    Length := Len;
    Post := Proc;
    DelayedSend := NBInt ($94, NCB) = 0;
  end;
end;

function SendDatagram;
begin
  with NCB do
  begin
    NameCopy (Remote, CallName);
    Buffer := @Buf;
    Length := Len;
    Post := nil;
    SendDatagram := not NBInt ($20, NCB) in [0, 9];
  end;
end;

function SendBroadDatagram;
begin
  with NCB do
  begin
    Buffer := @Buf;
    Length := Len;
    Post := nil;
    SendBroadDatagram := not NBInt ($A2, NCB) in [0, 9];
  end;
end;

function RECEIVE;
begin
  with NCB do
  begin
    LSN := aLSN;
    Buffer := @Buf;
    Length := Len;
    if NBInt ($15, NCB) = 0
    then RECEIVE := Len
    else RECEIVE := 0;
  end;
end;

function DELAYEDRECEIVE;
begin
  with NCB do
  begin
    LSN := aLSN;
    Buffer := @Buf;
    Length := Len;
    Post := Proc;
    if NBInt ($95, NCB) = 0
    then DelayedReceive := Len
    else DELAYEDRECEIVE := 0;
  end;
end;

function ReceiveDatagram;
begin
  with NCB do
  begin
    Buffer := @Buf;
    Length := Len;
    Post := Proc;
    ReceiveDatagram := NBInt ($A1, NCB) <> 0;
  end;
end;

function ReceiveBroadDatagram;
begin
  with NCB do
  begin
    Buffer := @Buf;
    Length := Len;
    Post := Proc;
    ReceiveBroadDatagram := not NBInt ($A3, NCB) in [0, 9];
  end;
end;

function HANGUP;
begin
  NCB.LSN := LSN;
  HANGUP := NBInt ($12, NCB) in [0, $A];
end;

end.

